(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('prop-types'), require('react'), require('react-dom/server')) :
  typeof define === 'function' && define.amd ? define(['prop-types', 'react', 'react-dom/server'], factory) :
  (global = global || self, global.ReactSVG = factory(global.PropTypes, global.React, global.ReactDOMServer));
}(this, function (PropTypes, React, ReactDOMServer) { 'use strict';

  ReactDOMServer = ReactDOMServer && ReactDOMServer.hasOwnProperty('default') ? ReactDOMServer['default'] : ReactDOMServer;

  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;

    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }

    return target;
  }

  var objectWithoutPropertiesLoose = _objectWithoutPropertiesLoose;

  function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    subClass.__proto__ = superClass;
  }

  var inheritsLoose = _inheritsLoose;

  var cloneSvg = function cloneSvg(sourceSvg) {
    return sourceSvg.cloneNode(true);
  };

  var isLocal = function isLocal() {
    return window.location.protocol === 'file:';
  };

  var svgCache = new Map();

  var requestQueue = {};
  var queueRequest = function queueRequest(url, callback) {
    requestQueue[url] = requestQueue[url] || [];
    requestQueue[url].push(callback);
  };
  var processRequestQueue = function processRequestQueue(url) {
    var _loop_1 = function _loop_1(i, len) {
      setTimeout(function () {
        if (Array.isArray(requestQueue[url])) {
          var cacheValue = svgCache.get(url);
          var callback = requestQueue[url][i];

          if (cacheValue instanceof SVGSVGElement || cacheValue instanceof HTMLElement) {
            callback(null, cloneSvg(cacheValue));
          }

          if (cacheValue instanceof Error) {
            callback(cacheValue);
          }

          if (i === requestQueue[url].length - 1) {
            delete requestQueue[url];
          }
        }
      }, 0);
    };

    for (var i = 0, len = requestQueue[url].length; i < len; i++) {
      _loop_1(i, len);
    }
  };

  var loadSvg = function loadSvg(url, callback) {
    if (svgCache.has(url)) {
      var cacheValue = svgCache.get(url);

      if (cacheValue instanceof SVGSVGElement || cacheValue instanceof HTMLElement) {
        callback(null, cloneSvg(cacheValue));
        return;
      }

      if (cacheValue instanceof Error) {
        callback(cacheValue);
        return;
      }

      queueRequest(url, callback);
      return;
    }

    svgCache.set(url, undefined);
    queueRequest(url, callback);
    var httpRequest = new XMLHttpRequest();

    httpRequest.onreadystatechange = function () {
      try {
        if (httpRequest.readyState === 4) {
          if (httpRequest.status === 404 || httpRequest.responseXML === null) {
            throw new Error(isLocal() ? 'Note: SVG injection ajax calls do not work locally without adjusting security setting in your browser. Or consider using a local webserver.' : 'Unable to load SVG file: ' + url);
          }

          if (httpRequest.status === 200 || isLocal() && httpRequest.status === 0) {
            if (httpRequest.responseXML instanceof Document) {
              if (httpRequest.responseXML.documentElement) {
                svgCache.set(url, httpRequest.responseXML.documentElement);
              }
            }

            processRequestQueue(url);
          } else {
            throw new Error('There was a problem injecting the SVG: ' + httpRequest.status + ' ' + httpRequest.statusText);
          }
        }
      } catch (error) {
        svgCache.set(url, error);
        processRequestQueue(url);
      }
    };

    httpRequest.open('GET', url);

    if (httpRequest.overrideMimeType) {
      httpRequest.overrideMimeType('text/xml');
    }

    httpRequest.send();
  };

  var idCounter = 0;

  var uniqueId = function uniqueId() {
    return ++idCounter;
  };

  var injectedElements = [];
  var ranScripts = {};
  var svgNamespace = 'http://www.w3.org/2000/svg';
  var xlinkNamespace = 'http://www.w3.org/1999/xlink';

  var injectElement = function injectElement(el, callback, _a) {
    var evalScripts = _a.evalScripts,
        renumerateIRIElements = _a.renumerateIRIElements;
    var imgUrl = el.getAttribute('data-src') || el.getAttribute('src');

    if (!imgUrl || !/\.svg/i.test(imgUrl)) {
      callback(new Error('Attempted to inject a file with a non-svg extension: ' + imgUrl));
      return;
    }

    if (injectedElements.indexOf(el) !== -1) {
      injectedElements.splice(injectedElements.indexOf(el), 1);
      el = null;
      return;
    }

    injectedElements.push(el);
    el.setAttribute('src', '');
    loadSvg(imgUrl, function (error, svg) {
      if (!svg) {
        injectedElements.splice(injectedElements.indexOf(el), 1);
        el = null;
        callback(error);
        return;
      }

      var imgId = el.getAttribute('id');

      if (imgId) {
        svg.setAttribute('id', imgId);
      }

      var imgTitle = el.getAttribute('title');

      if (imgTitle) {
        svg.setAttribute('title', imgTitle);
      }

      var imgWidth = el.getAttribute('width');

      if (imgWidth) {
        svg.setAttribute('width', imgWidth);
      }

      var imgHeight = el.getAttribute('height');

      if (imgHeight) {
        svg.setAttribute('height', imgHeight);
      }

      var mergedClasses = Array.from(new Set((svg.getAttribute('class') || '').split(' ').concat(['injected-svg'], (el.getAttribute('class') || '').split(' ')))).join(' ').trim();
      svg.setAttribute('class', mergedClasses);
      var imgStyle = el.getAttribute('style');

      if (imgStyle) {
        svg.setAttribute('style', imgStyle);
      }

      svg.setAttribute('data-src', imgUrl);
      var imgData = [].filter.call(el.attributes, function (at) {
        return /^data-\w[\w-]*$/.test(at.name);
      });
      Array.prototype.forEach.call(imgData, function (dataAttr) {
        if (dataAttr.name && dataAttr.value) {
          svg.setAttribute(dataAttr.name, dataAttr.value);
        }
      });

      if (renumerateIRIElements) {
        var iriElementsAndProperties_1 = {
          clipPath: ['clip-path'],
          'color-profile': ['color-profile'],
          cursor: ['cursor'],
          filter: ['filter'],
          linearGradient: ['fill', 'stroke'],
          marker: ['marker', 'marker-start', 'marker-mid', 'marker-end'],
          mask: ['mask'],
          path: [],
          pattern: ['fill', 'stroke'],
          radialGradient: ['fill', 'stroke']
        };
        var element_1;
        var elements_1;
        var properties_1;
        var currentId_1;
        var newId_1;
        Object.keys(iriElementsAndProperties_1).forEach(function (key) {
          element_1 = key;
          properties_1 = iriElementsAndProperties_1[key];
          elements_1 = svg.querySelectorAll(element_1 + '[id]');

          var _loop_1 = function _loop_1(a, elementsLen) {
            currentId_1 = elements_1[a].id;
            newId_1 = currentId_1 + '-' + uniqueId();
            var referencingElements;
            Array.prototype.forEach.call(properties_1, function (property) {
              referencingElements = svg.querySelectorAll('[' + property + '*="' + currentId_1 + '"]');

              for (var b = 0, referencingElementLen = referencingElements.length; b < referencingElementLen; b++) {
                var attrValue = referencingElements[b].getAttribute(property);

                if (attrValue && !attrValue.match(new RegExp('url\\(#' + currentId_1 + '\\)'))) {
                  continue;
                }

                referencingElements[b].setAttribute(property, 'url(#' + newId_1 + ')');
              }
            });
            var allLinks = svg.querySelectorAll('[*|href]');
            var links = [];

            for (var c = 0, allLinksLen = allLinks.length; c < allLinksLen; c++) {
              var href = allLinks[c].getAttributeNS(xlinkNamespace, 'href');

              if (href && href.toString() === '#' + elements_1[a].id) {
                links.push(allLinks[c]);
              }
            }

            for (var d = 0, linksLen = links.length; d < linksLen; d++) {
              links[d].setAttributeNS(xlinkNamespace, 'href', '#' + newId_1);
            }

            elements_1[a].id = newId_1;
          };

          for (var a = 0, elementsLen = elements_1.length; a < elementsLen; a++) {
            _loop_1(a, elementsLen);
          }
        });
      }

      svg.removeAttribute('xmlns:a');
      var scripts = svg.querySelectorAll('script');
      var scriptsToEval = [];
      var script;
      var scriptType;

      for (var i = 0, scriptsLen = scripts.length; i < scriptsLen; i++) {
        scriptType = scripts[i].getAttribute('type');

        if (!scriptType || scriptType === 'application/ecmascript' || scriptType === 'application/javascript' || scriptType === 'text/javascript') {
          script = scripts[i].innerText || scripts[i].textContent;

          if (script) {
            scriptsToEval.push(script);
          }

          svg.removeChild(scripts[i]);
        }
      }

      if (scriptsToEval.length > 0 && (evalScripts === 'always' || evalScripts === 'once' && !ranScripts[imgUrl])) {
        for (var l = 0, scriptsToEvalLen = scriptsToEval.length; l < scriptsToEvalLen; l++) {
          new Function(scriptsToEval[l])(window);
        }

        ranScripts[imgUrl] = true;
      }

      var styleTags = svg.querySelectorAll('style');
      Array.prototype.forEach.call(styleTags, function (styleTag) {
        styleTag.textContent += '';
      });
      svg.setAttribute('xmlns', svgNamespace);
      svg.setAttribute('xmlns:xlink', xlinkNamespace);

      if (el.parentNode) {
        el.parentNode.replaceChild(svg, el);
      }

      injectedElements.splice(injectedElements.indexOf(el), 1);
      el = null;
      callback(null, svg);
    });
  };

  var EvalScripts;

  (function (EvalScripts) {
    EvalScripts["Always"] = "always";
    EvalScripts["Once"] = "once";
    EvalScripts["Never"] = "never";
  })(EvalScripts || (EvalScripts = {}));

  var SVGInjector = function SVGInjector(elements, _a) {
    var _b = _a === void 0 ? {} : _a,
        _c = _b.done,
        done = _c === void 0 ? function () {
      return undefined;
    } : _c,
        _d = _b.each,
        each = _d === void 0 ? function () {
      return undefined;
    } : _d,
        _e = _b.evalScripts,
        evalScripts = _e === void 0 ? EvalScripts.Never : _e,
        _f = _b.renumerateIRIElements,
        renumerateIRIElements = _f === void 0 ? true : _f;

    if (elements && 'length' in elements) {
      var elementsLoaded_1 = 0;

      for (var i = 0, j = elements.length; i < j; i++) {
        injectElement(elements[i], function (error, svg) {
          each(error, svg);

          if (elements && 'length' in elements && elements.length === ++elementsLoaded_1) {
            done(elementsLoaded_1);
          }
        }, {
          evalScripts: evalScripts,
          renumerateIRIElements: renumerateIRIElements
        });
      }
    } else if (elements) {
      injectElement(elements, function (error, svg) {
        each(error, svg);
        done(1);
        elements = null;
      }, {
        evalScripts: evalScripts,
        renumerateIRIElements: renumerateIRIElements
      });
    } else {
      done(0);
    }
  };

  // Hat-tip: https://github.com/developit/preact-compat/blob/master/src/index.js#L402.
  var shallowDiffers = function shallowDiffers(a, b) {
    for (var i in a) {
      if (!(i in b)) {
        return true;
      }
    }

    for (var _i in b) {
      if (a[_i] !== b[_i]) {
        return true;
      }
    }

    return false;
  };

  var ReactSVG =
  /*#__PURE__*/
  function (_React$Component) {
    inheritsLoose(ReactSVG, _React$Component);

    function ReactSVG() {
      var _this;

      _this = _React$Component.apply(this, arguments) || this;
      _this.initialState = {
        hasError: false,
        isLoading: true
      };
      _this.state = _this.initialState;
      _this._isMounted = false;

      _this.refCallback = function (container) {
        _this.container = container;
      };

      return _this;
    }

    var _proto = ReactSVG.prototype;

    _proto.renderSVG = function renderSVG() {
      var _this2 = this;

      if (this.container instanceof Node) {
        var _this$props = this.props,
            evalScripts = _this$props.evalScripts,
            renumerateIRIElements = _this$props.renumerateIRIElements,
            src = _this$props.src,
            svgClassName = _this$props.svgClassName,
            svgStyle = _this$props.svgStyle;
        /* eslint-disable @typescript-eslint/no-non-null-assertion */

        var onInjected = this.props.onInjected;
        var Wrapper = this.props.wrapper;
        /* eslint-enable @typescript-eslint/no-non-null-assertion */

        var wrapper = document.createElement(Wrapper);
        wrapper.innerHTML = ReactDOMServer.renderToStaticMarkup(React.createElement(Wrapper, null, React.createElement(Wrapper, {
          className: svgClassName,
          "data-src": src,
          style: svgStyle
        })));
        this.svgWrapper = this.container.appendChild(wrapper.firstChild);

        var each = function each(error, svg) {
          if (error) {
            _this2.removeSVG();
          } // TODO: It'd be better to cleanly unsubscribe from SVGInjector
          // callbacks instead of tracking a property like this.


          if (_this2._isMounted) {
            _this2.setState(function () {
              return {
                hasError: !!error,
                isLoading: false
              };
            }, function () {
              onInjected(error, svg);
            });
          }
        };

        SVGInjector(this.svgWrapper.firstChild, {
          each: each,
          evalScripts: evalScripts,
          renumerateIRIElements: renumerateIRIElements
        });
      }
    };

    _proto.removeSVG = function removeSVG() {
      if (this.container instanceof Node && this.svgWrapper instanceof Node) {
        this.container.removeChild(this.svgWrapper);
        this.svgWrapper = null;
      }
    };

    _proto.componentDidMount = function componentDidMount() {
      this._isMounted = true;
      this.renderSVG();
    };

    _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
      var _this3 = this;

      if (shallowDiffers(prevProps, this.props)) {
        this.setState(function () {
          return _this3.initialState;
        }, function () {
          _this3.removeSVG();

          _this3.renderSVG();
        });
      }
    };

    _proto.componentWillUnmount = function componentWillUnmount() {
      this._isMounted = false;
      this.removeSVG();
    };

    _proto.render = function render() {
      /* eslint-disable @typescript-eslint/no-unused-vars */
      var _this$props2 = this.props,
          evalScripts = _this$props2.evalScripts,
          Fallback = _this$props2.fallback,
          Loading = _this$props2.loading,
          onInjected = _this$props2.onInjected,
          renumerateIRIElements = _this$props2.renumerateIRIElements,
          src = _this$props2.src,
          svgClassName = _this$props2.svgClassName,
          svgStyle = _this$props2.svgStyle,
          wrapper = _this$props2.wrapper,
          rest = objectWithoutPropertiesLoose(_this$props2, ["evalScripts", "fallback", "loading", "onInjected", "renumerateIRIElements", "src", "svgClassName", "svgStyle", "wrapper"]);
      /* eslint-enable @typescript-eslint/no-unused-vars */
      // eslint-disable-next-line @typescript-eslint/no-non-null-assertion


      var Wrapper = wrapper;
      return React.createElement(Wrapper, Object.assign({}, rest, {
        ref: this.refCallback
      }), this.state.isLoading && Loading && React.createElement(Loading, null), this.state.hasError && Fallback && React.createElement(Fallback, null));
    };

    return ReactSVG;
  }(React.Component);
  ReactSVG.defaultProps = {
    evalScripts: 'never',
    fallback: null,
    loading: null,
    onInjected: function onInjected() {
      return undefined;
    },
    renumerateIRIElements: true,
    svgClassName: null,
    svgStyle: {},
    wrapper: 'div'
  };
  ReactSVG.propTypes = {
    evalScripts: PropTypes.oneOf(['always', 'once', 'never']),
    fallback: PropTypes.oneOfType([PropTypes.func, PropTypes.object, PropTypes.string]),
    loading: PropTypes.oneOfType([PropTypes.func, PropTypes.object, PropTypes.string]),
    onInjected: PropTypes.func,
    renumerateIRIElements: PropTypes.bool,
    src: PropTypes.string.isRequired,
    svgClassName: PropTypes.string,
    svgStyle: PropTypes.object,
    wrapper: PropTypes.oneOf(['div', 'span'])
  };

  return ReactSVG;

}));
//# sourceMappingURL=react-svg.umd.development.js.map
